package main

import (
	"flag"
	"github.com/BurntSushi/toml"
	"log"
	"os"
)

var (
	configPath string
	read       bool
)

var config conf

type conf struct {
	System systemConf
	Web    webConf
}

type systemConf struct {
	StorePath           string
	FilePermission      int
	DirectoryPermission int
	BackupInterval      int
}

type webConf struct {
	Host  string
	Port  int
	Unix  bool
	Proxy bool
	Log   bool
}

func init() {
	flag.StringVar(&configPath, "c", "server.conf", "specify location of configuration file")
}

func configSetup() {
	if read {
		panic("config already read")
	}
	defer func() { read = true }()

	if meta, err := toml.DecodeFile(configPath, &config); err != nil {
		if !os.IsNotExist(err) {
			log.Fatalf("error parsing config: %s", err)
		}

		// generate
		var file *os.File
		if file, err = os.Create(configPath); err != nil {
			log.Fatalf("error creating config: %s", err)
		}
		if err = toml.NewEncoder(file).Encode(defConf); err != nil {
			log.Fatalf("error encoding config: %s", err)
		}
		config = defConf
		return
	} else {
		for _, key := range meta.Undecoded() {
			log.Printf("unused key in config: %s", key.String())
		}
	}
}

var defConf = conf{
	System: systemConf{
		StorePath:           "db",
		FilePermission:      0600,
		DirectoryPermission: 0700,
		BackupInterval:      0,
	},
	Web: webConf{
		Host:  "127.0.0.1",
		Port:  7777,
		Unix:  false,
		Proxy: true,
		Log:   false,
	},
}
