package main

import (
	"embed"
	"fmt"
	"github.com/gin-gonic/gin"
	"io/fs"
	"log"
	"net"
	"net/http"
	"os"
)

//go:embed assets
var assets embed.FS

var (
	server   = http.Server{}
	listener net.Listener
	router   *gin.Engine
)

func webSetup() {
	router = gin.New()
	router.Use(recovery())
	router.ForwardedByClientIP = config.Web.Proxy

	if config.Web.Log {
		router.Use(gin.Logger())
	}

	router.NoRoute(func(context *gin.Context) {
		context.Redirect(http.StatusTemporaryRedirect, "/web")
	})

	if config.Web.Unix {
		if l, err := net.Listen("unix", config.Web.Host); err != nil {
			log.Fatalf("error binding to %s: %s", config.Web.Host, err)
		} else {
			listener = l
		}
		log.Printf("listening on socket %s", config.Web.Host)

		if err := os.Chmod(config.Web.Host, 0777); err != nil {
			log.Printf("error chmod %s: %s", config.Web.Host, err)
		}
	} else {
		address := fmt.Sprintf("%s:%v", config.Web.Host, config.Web.Port)
		if l, err := net.Listen("tcp", address); err != nil {
			log.Fatalf("error binding to %s: %s", address, err)
		} else {
			listener = l
		}
		log.Printf("listening on %s", address)
	}

	server.Handler = router
}

func registerWebpage() {
	router.GET("/", func(context *gin.Context) {
		context.Redirect(http.StatusTemporaryRedirect, "web")
	})

	if stat, err := os.Stat("assets/public"); err == nil && stat.IsDir() {
		log.Println("serving assets from filesystem")
		router.Static("/web", "assets/public")
	} else {
		log.Println("serving bundle assets")
		var public fs.FS
		if public, err = fs.Sub(assets, "assets/public"); err != nil {
			log.Fatalf("error getting subtree: %s", err)
		}
		router.StaticFS("/web", http.FS(public))
	}
}

func runServer() {
	if err := server.Serve(listener); err != nil {
		if err == http.ErrServerClosed {
			log.Println("server closed")
		} else {
			log.Fatalf("error serve: %s", err)
		}
	}
}
